import ktc
import os
import onnx
from PIL import Image
import numpy as np

###  post process function  ###
import tensorflow as tf
import pathlib
import sys

###  pre process function  ###
def preprocess(pil_img):   
    image = tf.io.read_file(pil_img)
    image = tf.io.decode_jpeg(image, channels=3)
    image = tf.image.resize(image, [300, 300])
    image = tf.cast(image / 255., tf.float32)
    image = (image - 0.5) * 2;
    array = tf.Session().run(image)
    return array

# convert tflite model to onnx
print("(1) convert tflite model to onnx ====>")
m = ktc.onnx_optimizer.tflite2onnx_flow("/data1/benchmark_model/ssd_mobilenetv1.tflite", True)
m = ktc.onnx_optimizer.onnx2onnx_flow(m, eliminate_tail=True, opt_matmul=False, disable_fuse_bn=True)
onnx.save(m,'ssd_mobilenetv1_opt.tflite.onnx')

# setup ktc config
print("(2) setup ktc config ====>")
km = ktc.ModelConfig(1000, "0001", "720", onnx_model=m)

# npu(only) performance simulation
print("(3) npu(only) performance simulation ====>")
#eval_result = km.evaluate()
#print("\nNpu performance evaluation result:\n" + str(eval_result))

## onnx model check
print("(4) onnx model check ====>")
in_data = preprocess('/data1/ssd_mobilenet_v1/val2017_10/000000000139.jpg')
out_data = ktc.kneron_inference([in_data], onnx_file="/data1/benchmark_model/ssd_mobilenetv1_opt.tflite.onnx", input_names=["normalized_input_image_tensor"])
print(out_data)

# load and normalize all image data from folder
img_list = []
for (dir_path, _, file_names) in os.walk("/data1/ssd_mobilenet_v1/val2017_10"):
    for f_n in file_names:
        fullpath = os.path.join(dir_path, f_n)
        print("processing image: " + fullpath)
        img_data = preprocess(fullpath)
        img_list.append(img_data)

# fix point analysis
print("(5) fix point analysis ====>")
bie_model_path = km.analysis({"normalized_input_image_tensor": img_list})
print("\nFix point analysis done. Save bie model to '" + str(bie_model_path) + "'")

# bie model check
print("(6) bie model check ====>")
in_data = preprocess('/data1/ssd_mobilenet_v1/val2017_10/000000000139.jpg')
radix = ktc.get_radix(img_list)
out_data = ktc.kneron_inference([in_data], bie_file=bie_model_path, input_names=["normalized_input_image_tensor"], radix=radix, platform=720)
print(out_data)

# compile
print("(7) compile ====>")
nef_model_path = ktc.compile([km])
print("\nCompile done. Save Nef file to '" + str(nef_model_path) + "'")

# nef model check
print("(8) nef model check ====>")
in_data = preprocess('/data1/ssd_mobilenet_v1/val2017_10/000000000139.jpg')
radix = ktc.get_radix(img_list)
out_data = ktc.kneron_inference([in_data], nef_file=nef_model_path, radix=radix, platform=720)
print(out_data)

